
import requests
import collections


TIMESTEP = 0.005


class Snapshot(collections.namedtuple("_Snapshot", ["x", "y", "vx", "vy", "p",
                                                    "l", "r", "ml", "mu", "mr"])):

    @classmethod
    def from_response_lines(class_, lines, game):
        x, y = map(float, lines[1].split(" "))
        y -= 10  # We are given coordinates of the _center_ of the module
        vx, vy = map(float, lines[2].split(" "))
        p = float(lines[3])
        l, r = map(float, lines[4].split(" "))
        # To make the logic easier
        l += 10
        r -= 10
        return class_(
            x=x, y=y, vx=vx, vy=vy, p=p, l=l, r=r,
            ml=game.ml, mu=game.mu, mr=game.mr
        )

    def step(self, timestep=TIMESTEP):
        return Snapshot(
            x=(self.x + self.vx*timestep + 0.5*(6 * self.mr - 6 * self.ml)*timestep**2),
            y=(self.y + self.vy*timestep + 0.5*(-7 + 12 * self.mu)*timestep**2),
            vx=(self.vx + (6 * self.mr - 6 * self.ml)*timestep),
            vy=(self.vy + (-7 + 12 * self.mu)*timestep),
            p=(self.p - (0.5 * self.ml + 0.5 * self.mr + 1 * self.mu)*timestep),
            l=self.l,
            r=self.r,
            ml=(self.ml if self.p > 0 else 0),
            mr=(self.mr if self.p > 0 else 0),
            mu=(self.mu if self.p > 0 else 0)
        )

    def predict_landing(self):
        at = self
        while at.y > 0.0:
            at = at.step()
        return at

    def to_json(self):
        return {
            f: getattr(self, f) for f in self._fields
        }


class Game:

    def __init__(self, address="http://127.0.0.1:8000"):
        self.address = address
        self.id_ = None
        self.ml = 0
        self.mu = 0
        self.mr = 0

    def start(self):
        lines = requests.get(self.address + "/start").content.decode().split("\n")
        self.id_ = lines[0].strip()
        return Snapshot.from_response_lines(lines, self)

    def control(self, left, right, up):
        self.ml = left
        self.mr = right
        self.mu = up
        lines = requests.get(self.address + "/stav",
                             params={"id": self.id_,
                                     "nahoru": str(self.mu),
                                     "doleva": str(self.ml),
                                     "doprava": str(self.mr)}).content.decode().split("\n")
        if lines[0] in {"Krach!", "Zdar!"}:
            raise RuntimeError(lines[0])
        return Snapshot.from_response_lines(lines, self)
