
game = {
	id: null,
	width: 100,
	height: 100,
	motors: {
		up: 0,
		left: 0,
		right: 0
	}
};

KEY_TO_MOTOR = {
	87: "up",
	65: "left",
	68: "right"
}
event = null;

function start() {
	$(document).keydown(function(ev) {
		$("#motors #" + KEY_TO_MOTOR[ev.which]).css("background", "red");
		game.motors[KEY_TO_MOTOR[ev.which]] = 1;
	})
	$(document).keyup(function(ev) {
		$("#motors #" + KEY_TO_MOTOR[ev.which]).css("background", "blue");
		game.motors[KEY_TO_MOTOR[ev.which]] = 0;
	})
	$.get("/start", {}, function(data) {
		redraw(data);
		game.id = data.id;
		game.width = $(document).width();
		game.height = $(document).height();
		// Notice that we are using setTimeout and recalling later.
		// This is intentional as it allows gracefully decreasing refresh rate
		setTimeout(update(), 100);
	})
}

function linear_rescaler(inmin, inmax, outmin, outmax) {
	return function(x) {
		return (x - inmin) * (outmax - outmin) / (inmax - inmin) + outmin;
	}
}

function redraw(data) {
	mincoord = Math.min(0, data.x, data.y, data.l);
	maxcoord = Math.max(200, data.x + 10, data.y + 25, data.r + 10);
	if (maxcoord - mincoord < 200) {
		diff = (maxcoord - mincoord) / 2;
		mincoord -= diff;
		maxcoord += diff;
	}
	console.log(mincoord, maxcoord);
	rescalex = linear_rescaler(mincoord, maxcoord, 0, game.width);
	rescaley = linear_rescaler(maxcoord - mincoord, 0, 0, game.height);

	$("#lander").css("left", rescalex(data.x - 10))
		.css("width", Math.abs(rescalex(20) - rescalex(0)));
	$("#lander").css("top", rescaley(data.y + 20) - $("#zone").height())
		.css("height", Math.abs(rescaley(20) - rescaley(0)));

	$("#zone").css("top", rescaley(1.5))
		.css("left", rescalex(data.l - 10))
		.css("width", rescalex(data.r - data.l + 20))
		.css("height", Math.abs(rescaley(1.5) - rescaley(0)));

	$("#controls #fuel").css("width", data.p * 5)
		.css("background", data.p < 10 ? "red" : "yellow");
	$("#controls #vy").css("width", Math.abs(data.vy * 5))
		.css("background", data.vy < -5 ? "red" : "green");
	$("#controls #vx").css("width", Math.abs(data.vx * 5))
		.css("background", Math.abs(data.vx) > 5 ? "red" : "green");
}

function update() {
	$.get("/stav", {id: game.id, nahoru: game.motors.up,
					doleva: game.motors.left, doprava: game.motors.right},
		function(data) {
			if (data.result) {
				if (confirm(data.result + " Znova?")) {
					window.location.reload(true);
				}
			} else {
				redraw(data);
				setTimeout(update, 50);
			}
		}
	)
}
